//------------------------------------------------------------------------------
// Author Todd Stafney
// e-mail: todd@stafney.com
// url: http://stafney.com/
//
// Copyright (C) 2008 Todd M Stafney
//
// This file is part of BuildSystemPlugins.
//
// BuildSystemPlugins is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Text;
using ThoughtWorks.CruiseControl.Core;
using ThoughtWorks.CruiseControl.Core.Util;
using Exortech.NetReflector;

// CCNet requires plugins to be prefixed with "ccnet." and suffixed with ".plugin.dll"
namespace ccnet.Stafney.Labellers.plugin
{

    [ReflectorType("versionInfoLabeller")]
    public class VersionInfoLabeller : ILabeller
    {
        static private string WhereAmI()
        {
            System.Diagnostics.StackFrame sf = new System.Diagnostics.StackFrame(1, false);
            System.Reflection.MethodBase mb = sf.GetMethod();
            return string.Format("{0}.{1}", mb.DeclaringType.Name, mb.Name);
        }

        public VersionInfoLabeller()
        {
            Log.Debug(WhereAmI());
        }

        private string file = String.Empty;
        [ReflectorProperty("versionInfoFile", Required = true)]
        public string File
        {
            get { return file; }
            set { Log.Debug(String.Format("{0} setting File={1}", WhereAmI(), value)); file = value; }
        }

        private string attr = "AssemblyVersion";
        [ReflectorProperty("attribute", Required = false)]
        public string Attr
        {
            get { return attr; }
            set { Log.Debug(String.Format("{0} setting Attr={1}", WhereAmI(), value)); attr = value; }
        }

        private int count = 4;
        [ReflectorProperty("count", Required = false)]
        public int Count
        {
            get { return count; }
            set { Log.Debug(String.Format("{0} setting Count={1}", WhereAmI(), value)); count = value; }
        }

        private string prefix = String.Empty;
        [ReflectorProperty("prefix", Required = false)]
        public string Prefix
        {
            get { return prefix; }
            set { Log.Debug(String.Format("{0} setting Prefix={1}", WhereAmI(), value)); prefix = value; }
        }

        private string suffix = String.Empty;
        [ReflectorProperty("suffix", Required = false)]
        public string Suffix
        {
            get { return suffix; }
            set { Log.Debug(String.Format("{0} setting Suffix={0}", WhereAmI(), value)); suffix = value; }
        }

        static public Version GetVersion(System.IO.FileInfo file, string attribute)
        {
            using (System.IO.TextReader tr = new System.IO.StreamReader(file.FullName))
            {
                string reTxt = String.Format("{0}\\(\"(?<ver>.*)\"\\)", attribute);
                Log.Debug(String.Format("{0} using regular expression: {1}", WhereAmI(), reTxt));
                System.Text.RegularExpressions.Regex re =
                    new System.Text.RegularExpressions.Regex(reTxt);

                string txt = tr.ReadToEnd();
                Log.Debug(String.Format("{0} against text: {1}", WhereAmI(), txt));

                string guts = re.Match(txt).Groups["ver"].Value;
                Version ver = new Version(guts);

                Log.Debug(String.Format("{0} returning: {1}", WhereAmI(), ver));
                return ver;
            }
        }

        #region ILabeller Members

        public string Generate(IIntegrationResult integrationResult)
        {
            Log.Debug(WhereAmI());
            try
            {
                string viFile = File;
                if (!System.IO.Path.IsPathRooted(viFile))
                    viFile = System.IO.Path.Combine(integrationResult.WorkingDirectory, File);

                System.IO.FileInfo file = new System.IO.FileInfo(viFile);

                Log.Debug(String.Format("{0} geting version from {1}", WhereAmI(), file.FullName));

                Version ver = GetVersion(file, Attr);

                string result = string.Format("{0}{1}{2}", Prefix, ver.ToString(Count), Suffix);

                Log.Info(String.Format("{0} returning {1}", WhereAmI(), result));

                return result;
            }
            catch (Exception ex)
            {
                Log.Error("exception!");
                Log.Error(ex);
                throw;
            }
        }

        #endregion

        #region ITask Members

        public void Run(ThoughtWorks.CruiseControl.Core.IIntegrationResult result)
        {
            result.Label = Generate(result);
        }

        #endregion
    }
}
